------------------------------------------------------------------------------
--
-- Filename:                access.vhd
-- Inhalt:                  Beispielmodelle zu access-types:
--                          - acc_types : allgemeines Beispiel zu Zeigern
--                          - queue     : Warteschlange mit unvollstaendiger
--                                        Typdeklaration
--
------------------------------------------------------------------------------
--
-- Autoren:                 Gunther Lehmann, Bernhard Wunder  
-- Letzte Aenderung:        03.03.1994
--
------------------------------------------------------------------------------

ENTITY acc_types IS
END acc_types;

------------------------------------------------------------------------------

ARCHITECTURE behavioral OF acc_types IS
    
BEGIN

  PROCESS

    TYPE pos2 IS ARRAY (1 DOWNTO 0) OF positive;   -- (1)
    TYPE access_pos2 IS ACCESS pos2;               -- (2)

    VARIABLE p1 : access_pos2 := NEW pos2'(2,3);   -- (3)  Allocation, Initialisierung
    VARIABLE p2 : access_pos2 := NEW pos2;         -- (4)  Allocation, Defaultwert
    VARIABLE p3,p4 : access_pos2 := NULL;          -- (5)  keine Allocation

  BEGIN

    p2.ALL (0) := 8;                               -- (6)  Wertzuweisung an Objekt

    p2 (1) := 7;                                   -- (7)  Kurzvariante

    p3 := p2;                                      -- (8)  Adresszuweisung

    p4 := NEW pos2;                                -- (9)  Allocation

    p4.ALL := p2.ALL;                              -- (10) Wertzuweisung

    deallocate (p1);                               -- (11) Freigeben

    WAIT; 

  END PROCESS;

END behavioral;

------------------------------------------------------------------------------

CONFIGURATION acc_config OF acc_types IS

    FOR behavioral
    END FOR;

END acc_config;

------------------------------------------------------------------------------
------------------------------------------------------------------------------

PACKAGE queue_types IS

  TYPE queue_access IS (nul, add_element, delete_element, read_element);  
  -- Kommandos fuer Warteschlange

END queue_types;

------------------------------------------------------------------------------

USE work.queue_types.ALL;

ENTITY queue IS

  PORT (command : IN queue_access;
        value   : INOUT integer);

END queue;

------------------------------------------------------------------------------

ARCHITECTURE demo OF queue IS

  TYPE chain_element;
  TYPE pointer IS ACCESS chain_element;
  TYPE chain_element IS RECORD
    next_element : pointer;
    element_data : integer;
  END RECORD;

BEGIN

  handle_queue: PROCESS (command)

    VARIABLE empty_flag : boolean := true;
    VARIABLE first, last, help : pointer := NULL;

  BEGIN

    CASE command IS

--
--  Fuege Element hinzu!
--   

       WHEN add_element =>
        IF empty_flag THEN                           -- erstes Element hinzufuegen
          first := NEW chain_element;
          last := first;
          empty_flag := false;
        ELSE                                         -- weitere Elemente hinzufuegen
          last.next_element := NEW chain_element;
          last := last.next_element;
        END IF;
        last.element_data := value;

--
--  Lese erstes Element und loesche es!
--   

      WHEN delete_element =>
        IF empty_flag THEN                           -- Warteschlange ist leer
          ASSERT false REPORT "Empty queue!" SEVERITY note; 
        ELSE
          value <= first.element_data, 0 AFTER 10 ns;
          help := first;
          IF first = last THEN                       -- letztes Element wird gelesen
            empty_flag := true;
          ELSE                                       -- weitere Elemente vorhanden
            first := first.next_element;
          END IF;
          deallocate (help);
        END IF;

--
--  Lese erstes Element!
--   

      WHEN read_element =>
        IF empty_flag THEN                           -- Warteschlange ist leer
          ASSERT false REPORT "Empty queue!" SEVERITY note; 
        ELSE
          value <= first.element_data, 0 AFTER 10 ns;
        END IF;

--
--  Tue nichts!
--   

      WHEN nul =>
        NULL;

    END CASE;

  END PROCESS;

END demo;

------------------------------------------------------------------------------

CONFIGURATION queue_config OF queue IS

    FOR demo
    END FOR;

END queue_config;




