------------------------------------------------------------------------------
--
-- Filename:            barrel.vhd
-- Inhalt:              Modellierungsbeispiel einer Schiebelogik
--
------------------------------------------------------------------------------
--
-- Autor:               Manfred Selz, Gunther Lehmann
-- Letzte Aenderung:    16.02.1994
--
------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;   -- syntheseprogrammspezifisches Package

------------------------------------------------------------------------------

ENTITY barrel IS

  PORT (
    data_in:  IN  std_ulogic_vector (7 DOWNTO 0);
    data_out: OUT std_ulogic_vector (7 DOWNTO 0);
    adresse:  IN  std_logic_vector  (2 DOWNTO 0));

END barrel;

------------------------------------------------------------------------------

ARCHITECTURE eins OF barrel IS

  SIGNAL adr : integer RANGE 0 TO 7 := 0;

BEGIN

  adr <= conv_integer(adresse);                    -- Konvertierung in Integer

  PROCESS (data_in, adr)

  BEGIN

    -- Bei der Synthese kann die Verwendung des MOD-Operators
    -- in der Schleife zu unguenstigen Ergebnissen fuehren!
    -- Bessere Ergebnisse werden meist mit Puffervariablen erreicht (s.u.)

    FOR i IN 7 DOWNTO 0 LOOP
      data_out((i + adr) MOD 8) <= data_in(i);
    END LOOP;

  END PROCESS;

END eins;

------------------------------------------------------------------------------

ARCHITECTURE zwei OF barrel IS

BEGIN

  PROCESS(data_in, adresse)

    VARIABLE puffer : std_ulogic_vector (7 DOWNTO 0);

  BEGIN

    puffer := data_in;

    FOR i IN 0 TO 2 LOOP
      IF (adresse(i) = '1' ) THEN
        puffer := puffer(7-2**i DOWNTO 0) &
                  puffer(7 DOWNTO 7-2**i+1);
      END IF;
    END LOOP;

    data_out <= puffer;
  
  END PROCESS;

END zwei;

------------------------------------------------------------------------------

