------------------------------------------------------------------------------
--
-- Filename:            kette.vhd
-- Inhalt:              XOR-kette mit Variablen und Signalen 
--
------------------------------------------------------------------------------
--
-- Autor:               Manfred Selz, Gunther Lehmann
-- Letzte Aenderung:    16.02.1994
--
------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.all;

------------------------------------------------------------------------------

ENTITY kette IS

  PORT ( hbyte:  IN  std_ulogic_vector (0 TO 3) := "0000";
         value: OUT std_ulogic);

END kette;

------------------------------------------------------------------------------

ARCHITECTURE richtig OF kette IS

BEGIN

  PROCESS (hbyte)
    VARIABLE merker: std_ulogic := '0';
  BEGIN

    -- Durch die Verwendung von Variablen in der Schleife
    -- entsteht primaer eine Kette von XOR-Gattern

    merker := '0';

    FOR i IN hbyte'RANGE LOOP
      merker := merker XOR hbyte(i);
    END LOOP;
    value <= merker;
  END PROCESS;

END richtig;

------------------------------------------------------------------------------

ARCHITECTURE falsch OF kette IS

  SIGNAL merker: std_ulogic := '0';

BEGIN

  PROCESS (hbyte)
  BEGIN

    -- Durch die Verwendung von Signalen in der Schleife
    -- ueberschreiben sich die einzelnen Zuweisungen 
    -- Es entsteht bei der Synthese nur ein einzelnes XOR-Gatter

    FOR i IN hbyte'RANGE LOOP
      merker <= merker XOR hbyte(i);
    END LOOP;
  END PROCESS;

  value <= merker;

END falsch;

