------------------------------------------------------------------------------
--
-- Filename:                alu_p_bd.vhd
-- Inhalt:                  Package Body fuer die Uebungsaufgabe ALU
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder  
-- Letzte Aenderung:        20.01.1994
--
------------------------------------------------------------------------------

USE std.textio.all;

---------------------------------------------------------

PACKAGE BODY alu_pack IS

    CONSTANT ram_offset : bit_vector(7 DOWNTO 0) := "00001111";
    CONSTANT input_filename : string := "alu_comm";
    CONSTANT output_filename : string := "alu_rslt";

---------------------------------------------------------

    FUNCTION str_to_command (input : string(1 TO 6)) RETURN command IS
    BEGIN
        CASE input IS
            WHEN "con_ab" => RETURN con_ab;
            WHEN "add_ab" => RETURN add_ab;
            WHEN "add_of" => RETURN add_of;
            WHEN "mlt_ab" => RETURN mlt_ab;
            WHEN "sh_l_b" => RETURN sh_l_b;
            WHEN "sh_r_b" => RETURN sh_r_b;
            WHEN "sh_l_k" => RETURN sh_l_k;
            WHEN "sh_r_k" => RETURN sh_r_k;
            WHEN "flip_b" => RETURN flip_b;
            WHEN "flip_k" => RETURN flip_k;
            WHEN "turn_b" => RETURN turn_b;
            WHEN "turn_k" => RETURN turn_k;
            WHEN OTHERS   => ASSERT false REPORT "Unknown command!"
                                          SEVERITY failure;
        END CASE;
    END;

---------------------------------------------------------

    FUNCTION bit8_to_int (vector8 : bit_vector) RETURN integer IS
        VARIABLE result : integer := 0;
    BEGIN
        FOR i IN 0 TO vector8'LENGTH - 1 LOOP    -- Laenge nicht auf 8 bit begrenzt!
            IF vector8(i+vector8'LOW) = '1' THEN
                result := result + 2**i;
            END IF;
        END LOOP;
        RETURN result;
    END;

---------------------------------------------------------

    FUNCTION int_to_bit16 (int65535 : integer) RETURN bit_vector IS
        VARIABLE count : integer := -1 ;
        VARIABLE result : bit_vector(15 DOWNTO 0) ;
        VARIABLE int_val : integer := int65535;
    BEGIN
        ASSERT (int_val >= 0 AND int_val <= 65535) 
                         REPORT "Integer can't be represented by 16 bit" 
                         SEVERITY failure ;
        WHILE int_val > 0 LOOP
            count := count + 1;
            IF int_val MOD 2 = 1 THEN
                int_val := int_val - 1;
                result(count) := '1';
            END IF;
            int_val := int_val / 2;
        END LOOP;
        RETURN result;
    END ;

---------------------------------------------------------

    FUNCTION create_parity_16 (vector16 : bit_vector) RETURN bit IS
        VARIABLE count_1 : integer := 0;
     BEGIN
        count_loop : FOR i IN vector16'RANGE LOOP
            IF vector16(i) = '1' THEN
                count_1 := count_1 + 1;
            END IF;
        END LOOP;
        IF count_1 MOD 2 = 0 THEN
            RETURN '0';
        ELSE
            RETURN '1';
        END IF;
    END;

---------------------------------------------------------

    PROCEDURE check_parity_8 (SIGNAL checking8  : IN parity_8) IS
        VARIABLE count_1 : integer := 0;
    BEGIN
        count_loop : FOR i IN checking8.vector'RANGE LOOP
            IF checking8.vector(i) = '1' THEN
                count_1 := count_1 + 1;
            END IF;
        END LOOP;
        ASSERT ((count_1 MOD 2 = 0) AND (checking8.parity = '0')) 
            OR ((count_1 MOD 2 = 1) AND (checking8.parity = '1'))  
                               REPORT "Parity Error in Operand"
                               SEVERITY warning ;
    END;

---------------------------------------------------------

END alu_pack;
