------------------------------------------------------------------------------
--
-- Filename:                clock.vhd
-- Aufgabe:                 2.1.3
-- Inhalt:                  Erzeugung eines clock-Signals
--                          Ausgabe als Bit- und als std_ulogic-Signal
--                          Symmetrischer Takt, Taktfrequenz ueber
--                          GENERIC einstellbar
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder  
-- Letzte Aenderung:        04.02.1994
--
------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.all;

------------------------------------------------------------------------------

ENTITY clock IS

    GENERIC (clock_cycle : time := 100 ns);    -- Dauer eines Taktzyklus
    PORT (clk_bit        : OUT bit;            -- OUT-Ports koennen nicht
          clk_std_ulogic : OUT std_ulogic );   -- gelesen werden!

END clock;

------------------------------------------------------------------------------
-- interner Clock erforderlich, da von OUT-Ports nicht gelesen werden
-- kann. Alternative: Ports vom Typ BUFFER
-- Hier: Variante mit internen Signalen
------------------------------------------------------------------------------

ARCHITECTURE behavioral_sig OF clock IS

  SIGNAL clk_int_b : bit;
  SIGNAL clk_int_s : std_ulogic := '0'; -- muss initialisiert werden, damit
                                        -- Invertierung funktioniert, denn
                                        -- NOT('U') = 'U'     (Defaultwert)
  BEGIN

  clk_int_b <= NOT clk_int_b AFTER clock_cycle/2;
  clk_int_s <= NOT clk_int_s AFTER clock_cycle/2;

  clk_bit <= clk_int_b;
  clk_std_ulogic <= clk_int_s;

END behavioral_sig;

------------------------------------------------------------------------------
-- Variante mit Variablen fuer interne clocks
------------------------------------------------------------------------------

ARCHITECTURE behavioral_var OF clock IS

BEGIN

  clk_generation_bit : PROCESS

    VARIABLE clk_int_b : bit;

  BEGIN

    clk_bit <= clk_int_b;
    WAIT FOR clock_cycle/2;
    clk_int_b := NOT clk_int_b;
 
  END PROCESS;

-----------------------

  clk_generation_std_ulogic : PROCESS

    VARIABLE clk_int_s : std_ulogic := '0';  -- Notwendige Initialisierung (s.o.)

  BEGIN

    clk_std_ulogic <= clk_int_s;
    WAIT FOR clock_cycle/2;
    clk_int_s := NOT clk_int_s;

  END PROCESS;

END behavioral_var;


