------------------------------------------------------------------------------
--
-- Filename:                mux.vhd
-- Aufgabe:                 2.1.3
-- Inhalt:                  Beispiele verschiedener Multiplexer-Modelle
--                          mit Hilfe bedingter Signalzuweisungen
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder 
-- Letzte Aenderung:        07.02.94
--
------------------------------------------------------------------------------

ENTITY mux IS

  PORT (sig_a, sig_b, sig_c, sig_d : IN  bit;             -- 4 Eingangssignale
        sel                        : IN  bit_vector(1 TO 2) ;        -- Select
        sig_out                    : OUT bit ) ;           -- 1 Ausgangssignal
  
END mux ;

------------------------------------------------------------------------------
--
-- behavioral_1: Verwendung der nebenlaeufigen "conditional signal assignment"
--
------------------------------------------------------------------------------

ARCHITECTURE behavioral_1 OF mux IS

BEGIN

  sig_out <= sig_a WHEN sel = "00" ELSE
             sig_b WHEN sel = "01" ELSE
             sig_c WHEN sel = "10" ELSE
             sig_d ;

END behavioral_1 ;

------------------------------------------------------------------------------
--
-- behavioral_2: Verwendung der nebenlaeufigen "selected signal assignment"
--
------------------------------------------------------------------------------

ARCHITECTURE behavioral_2 OF mux IS

BEGIN

  WITH sel SELECT
    sig_out <= sig_a WHEN "00",
               sig_b WHEN "01",
               sig_c WHEN "10",
               sig_d WHEN OTHERS ;

END behavioral_2 ;

------------------------------------------------------------------------------
--
-- behavioral_3: Verwendung der sequentiellen IF-Anweisung
--
------------------------------------------------------------------------------

ARCHITECTURE behavioral_3 OF mux IS

BEGIN

  PROCESS (sig_a, sig_b, sig_c, sig_d, sel)

  BEGIN

    IF    sel = "00" THEN sig_out <= sig_a ;
    ELSIF sel = "01" THEN sig_out <= sig_b ;
    ELSIF sel = "10" THEN sig_out <= sig_c ;
    ELSE                  sig_out <= sig_d ;
    END IF ;

  END PROCESS ;

END behavioral_3 ;

------------------------------------------------------------------------------
--
-- behavioral_4: Verwendung der sequentiellen CASE-Anweisung
--
------------------------------------------------------------------------------

ARCHITECTURE behavioral_4 OF mux IS

BEGIN

  PROCESS (sig_a, sig_b, sig_c, sig_d, sel)

  BEGIN

    CASE sel IS
      WHEN "00"   => sig_out <= sig_a ;
      WHEN "01"   => sig_out <= sig_b ;
      WHEN "10"   => sig_out <= sig_c ;
      WHEN OTHERS => sig_out <= sig_d ;
    END CASE ;

  END PROCESS ;

END behavioral_4 ;

------------------------------------------------------------------------------

CONFIGURATION mux_config OF mux IS

  FOR behavioral_1
  END FOR ;

END mux_config ;
