------------------------------------------------------------------------------
--
-- Filename:                tab_gate.vhd
-- Inhalt:                  Durch Synthese von tab.vhd erzeugte 
--                          Gatternetzliste des Telefonanrufbeantworters
--                          verwendet 9-wertige Logik
--
------------------------------------------------------------------------------
--
-- Autor:                   syntheseprogramm 
-- Letzte Aenderung:        07.02.94
--
------------------------------------------------------------------------------


library IEEE;

use IEEE.std_logic_1164.all;

package CONV_PACK_tab_fsm is

-- define attributes
attribute ENUM_ENCODING : STRING;
   
   -- Declarations for conversion functions.
   function std_logic_to_BIT(arg : in std_logic) return BIT;
   function BIT_to_std_logic(arg : in BIT) return std_logic;

end CONV_PACK_tab_fsm;

package body CONV_PACK_tab_fsm is
   
   -- std_logic to enum type function
   function std_logic_to_BIT(arg : in std_logic) return BIT is
   -- synopsys built_in SYN_FEED_THRU;
   begin
      case arg is
         when '0' => return '0';
         when '1' => return '1';
         when others => assert FALSE -- this should not happen.
               report "un-convertible value"
               severity warning;
               return '0';
      end case;
   end;
   
   -- enum type to std_logic function
   function BIT_to_std_logic(arg : in BIT) return std_logic is
   -- synopsys built_in SYN_FEED_THRU;
   begin
      case arg is
         when '0' => return '0';
         when '1' => return '1';
         when others => assert FALSE -- this should not happen.
               report "un-convertible value"
               severity warning;
               return '0';
      end case;
   end;

end CONV_PACK_tab_fsm;

library IEEE;

use IEEE.std_logic_1164.all;

use work.CONV_PACK_tab_fsm.all;

entity tab_fsm is

   port( clk, reset, anruf, hoerer_aufgelegt, ansage_fertig, timer_ende : in 
         BIT;  timer_start, ansage_abspielen, anruf_aufzeichnen : out BIT);

end tab_fsm;

architecture SYN_behavioral of tab_fsm is

   component NR2
      port( A, B : in std_logic;  Z : out std_logic);
   end component;
   
   component NR3
      port( A, B, C : in std_logic;  Z : out std_logic);
   end component;
   
   component AO2
      port( A, B, C, D : in std_logic;  Z : out std_logic);
   end component;
   
   component AO4
      port( A, B, C, D : in std_logic;  Z : out std_logic);
   end component;
   
   component AO7
      port( A, B, C : in std_logic;  Z : out std_logic);
   end component;
   
   component IVA
      port( A : in std_logic;  Z : out std_logic);
   end component;
   
   component IV
      port( A : in std_logic;  Z : out std_logic);
   end component;
   
   component ND2
      port( A, B : in std_logic;  Z : out std_logic);
   end component;
   
   component ND4
      port( A, B, C, D : in std_logic;  Z : out std_logic);
   end component;
   
   component FD2
      port( D, CP, CD : in std_logic;  Q, QN : out std_logic);
   end component;
   
   signal anruf_port, net8, net9, reset_port, state_1_port, n320, n321, n322
      , n323, clk_port, timer_ende_port, ansage_abspielen_port, state_2_port, 
      hoerer_aufgelegt_port, net10, anruf_aufzeichnen_port, ansage_fertig_port,
      state_0_port, timer_start_port, n312, n313, n314, n315, n316, n317, n318,
      n319 : std_logic;

begin
   clk_port <= BIT_to_std_logic(clk);
   reset_port <= BIT_to_std_logic(reset);
   anruf_port <= BIT_to_std_logic(anruf);
   hoerer_aufgelegt_port <= BIT_to_std_logic(hoerer_aufgelegt);
   ansage_fertig_port <= BIT_to_std_logic(ansage_fertig);
   timer_ende_port <= BIT_to_std_logic(timer_ende);
   timer_start <= std_logic_to_BIT(timer_start_port);
   ansage_abspielen <= std_logic_to_BIT(ansage_abspielen_port);
   anruf_aufzeichnen <= std_logic_to_BIT(anruf_aufzeichnen_port);
   
   U129 : AO7  port map( A => ansage_fertig_port, B => net9, C => net10, Z 
                           => n319);
   U128 : AO7  port map( A => state_2_port, B => n312, C => state_1_port, Z 
                           => n314);
   U127 : ND4  port map( A => timer_ende_port, B => n312, C => state_0_port,
                           D => net10, Z => n315);
   U126 : NR2  port map( A => net9, B => net8, Z => anruf_aufzeichnen_port);
   U125 : AO2  port map( A => timer_ende_port, B => state_0_port, C => n319,
                           D => net8, Z => n318);
   U124 : ND2  port map( A => anruf_port, B => hoerer_aufgelegt_port, Z => 
                           n317);
   U123 : NR2  port map( A => anruf_aufzeichnen_port, B => n318, Z => n316);
   U136 : AO4  port map( A => net8, B => net10, C => n316, D => n317, Z => 
                           n320);
   U122 : NR3  port map( A => state_1_port, B => anruf_port, C => 
                           state_0_port, Z => n313);
   U135 : ND2  port map( A => n314, B => n315, Z => n321);
   U134 : AO7  port map( A => n313, B => net10, C => hoerer_aufgelegt_port, 
                           Z => n323);
   U133 : NR2  port map( A => state_0_port, B => net9, Z => 
                           ansage_abspielen_port);
   U132 : NR2  port map( A => state_1_port, B => net8, Z => timer_start_port
                           );
   U131 : IVA  port map( A => reset_port, Z => n322);
   U130 : IV  port map( A => n317, Z => n312);
   state_reg_2_label : FD2  port map( D => n323, CP => clk_port, CD => n322,
                           Q => state_2_port, QN => net10);
   state_reg_1_label : FD2  port map( D => n321, CP => clk_port, CD => n322,
                           Q => state_1_port, QN => net9);
   state_reg_0_label : FD2  port map( D => n320, CP => clk_port, CD => n322,
                           Q => state_0_port, QN => net8);

end SYN_behavioral;
